/*
 *  Copyright (c) 2016 EMC Corporation All Rights Reserved
 *
 *  This software contains the intellectual property of EMC
 *  Corporation or is licensed to EMC Corporation from third
 *  parties. Use of this software and the intellectual property
 *  contained therein is expressly limited to the terms and
 *  conditions of the License Agreement under which it is
 *  provided by or on behalf of EMC.
 */
package org.mark.demo;

import java.util.concurrent.atomic.AtomicInteger;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import akka.actor.UntypedActor;
import akka.routing.FromConfig;

import static org.mark.demo.SpringExtension.SpringExtProvider;

@Component
@Scope("prototype")
public abstract class BaseActor extends UntypedActor {
    private static final Logger logger = LoggerFactory.getLogger(BaseActor.class);

    private static AtomicInteger instantiationCount = new AtomicInteger(0);

    @Autowired
    private ActorSystem system;

    @Override
    public void postStop() {
        logger.info("ACTOR STOPPED. Path: {}, Instantiation count {}", self().path(), instantiationCount);
    }

    @Override
    public void preStart() throws Exception {
        logger.info("ACTOR STARTED. Path {}, Instantiation count {}", self().path(), instantiationCount);
        super.preStart();
    }

    public BaseActor() {
        logger.info("ACTOR CREATED. Instantiation count {}", instantiationCount.getAndIncrement());

    }

    @Override
    protected void finalize() {
        logger.info("ACTOR FINALIZED. Instantiation count {}", instantiationCount.getAndDecrement());
    }

    protected void sendMessage(String actorBeanName, String actorRouterName, Object message) {
        ActorRef actor = null;
        final scala.Option<ActorRef> child = context().child(actorRouterName);
        if (child != null && child.isDefined()) {
            actor = child.get();
            //logger.info("Will be reusing existing actor {} with the parent path {}", actorRouterName, self().path());
        } else {
            //logger.info("Will create a new actor {} with the parent path {}", actorRouterName, self().path());
            actor = getContext().actorOf(SpringExtProvider.get(system).props(actorBeanName).withRouter(new FromConfig()), actorRouterName);
        }

        actor.tell(message, self());
    }

}
